@@ -23,7 +23,7 @@ from dask.utils import ignoring, tmpfile, tmpdir
 from dask.utils_test import inc
 
 from dask.array import chunk
-from dask.array.core import (getem, getarray, getarray_nofancy, top, dotmany,
+from dask.array.core import (getem, getarray, top, dotmany,
                              concatenate3, broadcast_dimensions, Array, stack,
                              concatenate, from_array, take, elemwise, isnull,
                              notnull, broadcast_shapes, partial_by_order,
@@ -54,12 +54,11 @@ def same_keys(a, b):
 
 
 def test_getem():
-    for fancy, getter in [(True, getarray), (False, getarray_nofancy)]:
-        sol = {('X', 0, 0): (getter, 'X', (slice(0, 2), slice(0, 3))),
-               ('X', 1, 0): (getter, 'X', (slice(2, 4), slice(0, 3))),
-               ('X', 1, 1): (getter, 'X', (slice(2, 4), slice(3, 6))),
-               ('X', 0, 1): (getter, 'X', (slice(0, 2), slice(3, 6)))}
-        assert getem('X', (2, 3), shape=(4, 6), fancy=fancy) == sol
+    sol = {('X', 0, 0): (getarray, 'X', (slice(0, 2), slice(0, 3))),
+           ('X', 1, 0): (getarray, 'X', (slice(2, 4), slice(0, 3))),
+           ('X', 1, 1): (getarray, 'X', (slice(2, 4), slice(3, 6))),
+           ('X', 0, 1): (getarray, 'X', (slice(0, 2), slice(3, 6)))}
+    assert getem('X', (2, 3), shape=(4, 6)) == sol
 
 
 def test_top():
@@ -277,6 +276,15 @@ def test_stack_scalars():
     assert s.compute().tolist() == [np.arange(4).mean(), np.arange(4).sum()]
 
 
+def test_stack_promote_type():
+    i = np.arange(10, dtype='i4')
+    f = np.arange(10, dtype='f4')
+    di = da.from_array(i, chunks=5)
+    df = da.from_array(f, chunks=5)
+    res = da.stack([di, df])
+    assert_eq(res, np.stack([i, f]))
+
+
 @pytest.mark.skipif(LooseVersion(np.__version__) < '1.10.0',
                     reason="NumPy doesn't yet support stack")
 def test_stack_rechunk():
@@ -319,6 +327,35 @@ def test_concatenate():
     pytest.raises(ValueError, lambda: concatenate([a, b, c], axis=2))
 
 
+def test_concatenate_unknown_axes():
+    dd = pytest.importorskip('dask.dataframe')
+    pd = pytest.importorskip('pandas')
+
+    a_df = pd.DataFrame({'x': np.arange(12)})
+    b_df = pd.DataFrame({'y': np.arange(12) * 10})
+
+    a_ddf = dd.from_pandas(a_df, sort=False, npartitions=3)
+    b_ddf = dd.from_pandas(b_df, sort=False, npartitions=3)
+
+    a_x = a_ddf.values
+    b_x = b_ddf.values
+
+    assert np.isnan(a_x.shape[0])
+    assert np.isnan(b_x.shape[0])
+
+    da.concatenate([a_x, b_x], axis=0)  # works fine
+
+    with pytest.raises(ValueError) as exc_info:
+        da.concatenate([a_x, b_x], axis=1)  # unknown chunks
+
+    assert 'nan' in str(exc_info.value)
+    assert 'allow_unknown_chunksize' in str(exc_info.value)
+
+    c_x = da.concatenate([a_x, b_x], axis=1, allow_unknown_chunksizes=True)  # unknown chunks
+
+    assert_eq(c_x, np.concatenate([a_df.values, b_df.values], axis=1))
+
+
 def test_concatenate_rechunk():
     x = da.random.random((6, 6), chunks=(3, 3))
     y = da.random.random((6, 6), chunks=(2, 2))
@@ -563,6 +600,23 @@ def test_tensordot():
     # assert_eq(tensordot(a, a), np.tensordot(x, x))
 
 
+@pytest.mark.parametrize('axes', [
+    0,
+    1,
+    (0, 1),
+    (1, 0),
+    ((1, 0), (2, 1)),
+    ((1, 2), (2, 0)),
+    ((2, 0), (1, 2))
+])
+def test_tensordot_2(axes):
+    x = np.arange(4 * 4 * 4).reshape((4, 4, 4))
+    y = da.from_array(x, chunks=2)
+
+    assert_eq(da.tensordot(y, y, axes=axes),
+              np.tensordot(x, x, axes=axes))
+
+
 def test_dot_method():
     x = np.arange(400).reshape((20, 20))
     a = from_array(x, chunks=(5, 5))
@@ -712,6 +766,33 @@ def test_ravel():
     assert_eq(np.ravel(x), da.ravel(a))
 
 
+def _maybe_len(l):
+    try:
+        return len(l)
+    except TypeError:
+        return 0
+
+
+@pytest.mark.parametrize('chunks', [(4, 6), (2, 6)])
+@pytest.mark.parametrize('shift', [3, 7, 9, (3, 9), (7, 2)])
+@pytest.mark.parametrize('axis', [None, 0, 1, -1, (0, 1), (1, 0)])
+def test_roll(chunks, shift, axis):
+    x = np.random.randint(10, size=(4, 6))
+    a = from_array(x, chunks=chunks)
+
+    if _maybe_len(shift) != _maybe_len(axis):
+        with pytest.raises(TypeError if axis is None else ValueError):
+            da.roll(a, shift, axis)
+    else:
+        if (_maybe_len(shift) > 1 and
+                LooseVersion(np.__version__) < LooseVersion("1.12.0")):
+            pytest.skip(
+                "NumPy %s doesn't support multiple axes with `roll`."
+                " Need NumPy 1.12.0 or greater." % np.__version__
+            )
+        assert_eq(np.roll(x, shift, axis), da.roll(a, shift, axis))
+
+
 @pytest.mark.parametrize('original_shape,new_shape,chunks', [
     ((10,), (10,), (3, 3, 4)),
     ((10,), (10, 1, 1), 5),
@@ -744,11 +825,22 @@ def test_ravel():
     ((4, 64), (4, 8, 4, 2), (2, 16)),
     ((4, 8, 4, 2), (2, 1, 2, 32, 2), (2, 4, 2, 2)),
     ((4, 1, 4), (4, 4), (2, 1, 2)),
+    ((0, 10), (0, 5, 2), (5, 5)),
+    ((5, 0, 2), (0, 10), (5, 2, 2)),
+    ((0,), (2, 0, 2), (4,)),
+    ((2, 0, 2), (0,), (4, 4, 4)),
 ])
 def test_reshape(original_shape, new_shape, chunks):
     x = np.random.randint(10, size=original_shape)
     a = from_array(x, chunks=chunks)
-    assert_eq(x.reshape(new_shape), a.reshape(new_shape))
+
+    xr = x.reshape(new_shape)
+    ar = a.reshape(new_shape)
+
+    if a.shape == new_shape:
+        assert a is ar
+
+    assert_eq(xr, ar)
 
 
 def test_reshape_exceptions():
@@ -851,8 +943,10 @@ def test_map_blocks_with_constants():
 def test_map_blocks_with_kwargs():
     d = da.arange(10, chunks=5)
 
-    assert_eq(d.map_blocks(np.max, axis=0, keepdims=True, dtype=d.dtype),
-              np.array([4, 9]))
+    result = d.map_blocks(np.max, axis=0, keepdims=True, dtype=d.dtype,
+                          chunks=(1,))
+
+    assert_eq(result, np.array([4, 9]))
 
 
 def test_map_blocks_with_chunks():
@@ -957,6 +1051,36 @@ def test_coerce():
         assert complex(d)
 
 
+def test_store_delayed_target():
+    from dask.delayed import delayed
+    d = da.ones((4, 4), chunks=(2, 2))
+    a, b = d + 1, d + 2
+
+    # empty buffers to be used as targets
+    targs = {}
+
+    def make_target(key):
+        a = np.empty((4, 4))
+        targs[key] = a
+        return a
+
+    # delayed calls to these targets
+    atd = delayed(make_target)('at')
+    btd = delayed(make_target)('bt')
+
+    store([a, b], [atd, btd])
+
+    at = targs['at']
+    bt = targs['bt']
+
+    assert_eq(at, a)
+    assert_eq(bt, b)
+
+    pytest.raises(ValueError, lambda: store([a], [at, bt]))
+    pytest.raises(ValueError, lambda: store(at, at))
+    pytest.raises(ValueError, lambda: store([at, bt], [at, bt]))
+
+
 def test_store():
     d = da.ones((4, 4), chunks=(2, 2))
     a, b = d + 1, d + 2
@@ -1445,12 +1569,6 @@ def test_Array_normalizes_dtype():
     assert isinstance(x.dtype, np.dtype)
 
 
-def test_args():
-    x = da.ones((10, 2), chunks=(3, 1), dtype='i4') + 1
-    y = Array(*x._args)
-    assert_eq(x, y)
-
-
 def test_from_array_with_lock():
     x = np.arange(10)
     d = da.from_array(x, chunks=5, lock=True)
@@ -1480,6 +1598,21 @@ def test_from_array_slicing_results_in_ndarray():
     assert type(s3.compute()) == np.ndarray
 
 
+def test_from_array_getitem():
+    x = np.arange(10)
+
+    def my_getitem(x, ind):
+        return x[ind]
+
+    y = da.from_array(x, chunks=(5,), getitem=my_getitem)
+
+    for k, v in y.dask.items():
+        if isinstance(v, tuple):
+            assert v[0] is my_getitem
+
+    assert_eq(x, y)
+
+
 def test_asarray():
     assert_eq(da.asarray([1, 2, 3]), np.asarray([1, 2, 3]))
 
@@ -1996,9 +2129,9 @@ def test_map_blocks_with_changed_dimension():
     with pytest.raises(ValueError):
         d.map_blocks(lambda b: b.sum(axis=1), drop_axis=1, dtype=d.dtype)
 
-    # Can't use both drop_axis and new_axis
+    # Adding axis with a gap
     with pytest.raises(ValueError):
-        d.map_blocks(lambda b: b, drop_axis=1, new_axis=1)
+        d.map_blocks(lambda b: b, new_axis=(3, 4))
 
     d = da.from_array(x, chunks=(4, 8))
     e = d.map_blocks(lambda b: b.sum(axis=1), drop_axis=1, dtype=d.dtype)
@@ -2017,6 +2150,19 @@ def test_map_blocks_with_changed_dimension():
     assert e.chunks == ((1,), (4, 4), (4, 4), (1,))
     assert_eq(e, x[None, :, :, None])
 
+    # Both new_axis and drop_axis
+    d = da.from_array(x, chunks=(8, 4))
+    e = d.map_blocks(lambda b: b.sum(axis=0)[:, None, None],
+                     drop_axis=0, new_axis=(1, 2), dtype=d.dtype)
+    assert e.chunks == ((4, 4), (1,), (1,))
+    assert_eq(e, x.sum(axis=0)[:, None, None])
+
+    d = da.from_array(x, chunks=(4, 8))
+    e = d.map_blocks(lambda b: b.sum(axis=1)[:, None, None],
+                     drop_axis=1, new_axis=(1, 2), dtype=d.dtype)
+    assert e.chunks == ((4, 4), (1,), (1,))
+    assert_eq(e, x.sum(axis=1)[:, None, None])
+
 
 def test_broadcast_chunks():
     assert broadcast_chunks(((5, 5),), ((5, 5),)) == ((5, 5),)
@@ -2439,7 +2585,7 @@ def test_atop_concatenate():
 
     z = atop(f, 'j', x, 'ijk', y, 'ki', y, 'ij', concatenate=True,
              dtype=x.dtype)
-    assert_eq(z, np.ones(10))
+    assert_eq(z, np.ones(10), check_shape=False)
 
 
 def test_common_blockdim():
@@ -2553,6 +2699,44 @@ def test_repeat():
         assert all(concat(d.repeat(r).chunks))
 
 
+@pytest.mark.parametrize('shape, chunks', [
+    ((10,), (1,)),
+    ((10, 11, 13), (4, 5, 3)),
+])
+@pytest.mark.parametrize('reps', [0, 1, 2, 3, 5])
+def test_tile(shape, chunks, reps):
+    x = np.random.random(shape)
+    d = da.from_array(x, chunks=chunks)
+
+    assert_eq(np.tile(x, reps), da.tile(d, reps))
+
+
+@pytest.mark.parametrize('shape, chunks', [
+    ((10,), (1,)),
+    ((10, 11, 13), (4, 5, 3)),
+])
+@pytest.mark.parametrize('reps', [-1, -5])
+def test_tile_neg_reps(shape, chunks, reps):
+    x = np.random.random(shape)
+    d = da.from_array(x, chunks=chunks)
+
+    with pytest.raises(ValueError):
+        da.tile(d, reps)
+
+
+@pytest.mark.parametrize('shape, chunks', [
+    ((10,), (1,)),
+    ((10, 11, 13), (4, 5, 3)),
+])
+@pytest.mark.parametrize('reps', [[1], [1, 2]])
+def test_tile_array_reps(shape, chunks, reps):
+    x = np.random.random(shape)
+    d = da.from_array(x, chunks=chunks)
+
+    with pytest.raises(NotImplementedError):
+        da.tile(d, reps)
+
+
 def test_concatenate_stack_dont_warn():
     with warnings.catch_warnings(record=True) as record:
         da.concatenate([da.ones(2, chunks=1)] * 62)
@@ -2769,11 +2953,11 @@ def test_fast_from_array():
 
 
 def test_random_from_array():
-    x = np.zeros(10000000)
+    x = np.zeros(500000000)
     start = time.time()
     y = da.from_array(x, chunks=x.shape[0] / 10, name=False)
     end = time.time()
-    assert end - start < 0.100
+    assert end - start < 0.400
 
     y2 = da.from_array(x, chunks=x.shape[0] / 10, name=False)
     assert y.name != y2.name
@@ -2816,3 +3000,37 @@ def test_atop_with_numpy_arrays():
 
     s = da.sum(x)
     assert any(x is v for v in s.dask.values())
+
+
+@pytest.mark.parametrize('chunks', (100, 6))
+@pytest.mark.parametrize('other', [[0, 0, 1], [2, 1, 3], (0, 0, 1)])
+def test_elemwise_with_lists(chunks, other):
+    x = np.arange(12).reshape((4, 3))
+    d = da.arange(12, chunks=chunks).reshape((4, 3))
+
+    x2 = np.vstack([x[:, 0], x[:, 1], x[:, 2]]).T
+    d2 = da.vstack([d[:, 0], d[:, 1], d[:, 2]]).T
+
+    assert_eq(x2, d2)
+
+    x3 = x2 * other
+    d3 = d2 * other
+
+    assert_eq(x3, d3)
+
+
+def test_constructor_plugin():
+    L = []
+    L2 = []
+    with dask.set_options(array_plugins=[L.append, L2.append]):
+        x = da.ones(10, chunks=5)
+        y = x + 1
+
+    assert L == L2 == [x, y]
+
+    with dask.set_options(array_plugins=[lambda x: x.compute()]):
+        x = da.ones(10, chunks=5)
+        y = x + 1
+
+    assert isinstance(y, np.ndarray)
+    assert len(L) == 2
