@@ -32,10 +32,10 @@ except:
 
 from ..base import Base, normalize_token, tokenize
 from ..compatibility import apply, urlopen
-from ..context import _globals
+from ..context import _globals, defer_to_globals
 from ..core import quote, istask, get_dependencies, reverse_dict
 from ..multiprocessing import get as mpget
-from ..optimize import fuse, cull, inline
+from ..optimize import fuse, cull, inline, dont_optimize
 from ..utils import (open, system_encoding, takes_multiple_arguments, funcname,
                      digit, insert)
 from ..bytes.core import write_bytes
@@ -105,6 +105,7 @@ def inline_singleton_lists(dsk, dependencies=None):
     return dsk
 
 
+@defer_to_globals('bag_optimize', falsey=dont_optimize)
 def optimize(dsk, keys, fuse_keys=None, rename_fused_keys=True, **kwargs):
     """ Optimize a dask from a dask.bag """
     dsk2, dependencies = cull(dsk, keys)
@@ -1099,11 +1100,13 @@ class Bag(Base):
         else:
             head = self.take(1)[0]
             meta = pd.DataFrame([head], columns=columns)
-        columns = list(meta.columns)
-        name = 'to_dataframe-' + tokenize(self, columns)
-        DataFrame = partial(pd.DataFrame, columns=columns)
-        dsk = dict(((name, i), (DataFrame, (list2, (self.name, i))))
-                   for i in range(self.npartitions))
+        # Serializing the columns and dtypes is much smaller than serializing
+        # the empty frame
+        cols = list(meta.columns)
+        dtypes = meta.dtypes.to_dict()
+        name = 'to_dataframe-' + tokenize(self, cols, dtypes)
+        dsk = {(name, i): (to_dataframe, (list2, (self.name, i)), cols, dtypes)
+               for i in range(self.npartitions)}
 
         divisions = [None] * (self.npartitions + 1)
         return dd.DataFrame(merge(optimize(self.dask, self._keys()), dsk),
@@ -1578,10 +1581,15 @@ def groupby_disk(b, grouper, npartitions=None, blocksize=2**20):
 
     import partd
     p = ('partd-' + token,)
+    dirname = _globals.get('temporary_directory', None)
+    if dirname:
+        file = (apply, partd.File, (), {'dir': dirname})
+    else:
+        file = (partd.File,)
     try:
-        dsk1 = {p: (partd.Python, (partd.Snappy, (partd.File,)))}
+        dsk1 = {p: (partd.Python, (partd.Snappy, file))}
     except AttributeError:
-        dsk1 = {p: (partd.Python, (partd.File,))}
+        dsk1 = {p: (partd.Python, file)}
 
     # Partition data on disk
     name = 'groupby-part-{0}-{1}'.format(funcname(grouper), token)
@@ -1684,3 +1692,9 @@ def split(seq, n):
     L = [seq[int(part * i): int(part * (i + 1))] for i in range(n - 1)]
     L.append(seq[int(part * (n - 1)):])
     return L
+
+
+def to_dataframe(seq, columns, dtypes):
+    import pandas as pd
+    res = pd.DataFrame(seq, columns=columns)
+    return res.astype(dtypes, copy=False)
