@@ -834,10 +834,17 @@ class _GroupBy(object):
             else:
                 group_columns = set()
 
-            # NOTE: this step relies on the index normalization to replace
-            #       series with their name in an index.
-            non_group_columns = [col for col in self.obj.columns
-                                 if col not in group_columns]
+            if self._slice:
+                # pandas doesn't exclude the grouping column in a SeriesGroupBy
+                # like df.groupby('a')['a'].agg(...)
+                non_group_columns = self._slice
+                if not isinstance(non_group_columns, list):
+                    non_group_columns = [non_group_columns]
+            else:
+                # NOTE: this step relies on the index normalization to replace
+                #       series with their name in an index.
+                non_group_columns = [col for col in self.obj.columns
+                                     if col not in group_columns]
 
             spec = _normalize_spec(arg, non_group_columns)
 
@@ -859,14 +866,6 @@ class _GroupBy(object):
         else:
             levels = 0
 
-        # apply the transformations to determine the meta object
-        meta_groupby = pd.Series([], dtype=bool, index=self.obj._meta.index)
-        meta_stage1 = _groupby_apply_funcs(self.obj._meta, funcs=chunk_funcs,
-                                           by=meta_groupby)
-        meta_stage2 = _groupby_apply_funcs(meta_stage1, funcs=aggregate_funcs,
-                                           level=0)
-        meta = _agg_finalize(meta_stage2, finalizers)
-
         if not isinstance(self.index, list):
             chunk_args = [self.obj, self.index]
 
@@ -880,11 +879,11 @@ class _GroupBy(object):
                   aggregate_kwargs=dict(funcs=aggregate_funcs, level=levels),
                   combine=_groupby_apply_funcs,
                   combine_kwargs=dict(funcs=aggregate_funcs, level=levels),
-                  meta=meta, token='aggregate', split_every=split_every,
+                  token='aggregate', split_every=split_every,
                   split_out=split_out, split_out_setup=split_out_on_index)
 
-        return map_partitions(_agg_finalize, obj, meta=meta,
-                              token='aggregate-finalize', funcs=finalizers)
+        return map_partitions(_agg_finalize, obj, token='aggregate-finalize',
+                              funcs=finalizers)
 
     @insert_meta_param_description(pad=12)
     def apply(self, func, meta=no_default, columns=no_default):
@@ -1036,10 +1035,6 @@ class SeriesGroupBy(_GroupBy):
 
     def nunique(self, split_every=None, split_out=1):
         name = self._meta.obj.name
-        meta = pd.Series([], dtype='int64',
-                         index=pd.Index([], dtype=self._meta.obj.dtype),
-                         name=name)
-
         levels = _determine_levels(self.index)
 
         if isinstance(self.obj, DataFrame):
@@ -1052,7 +1047,7 @@ class SeriesGroupBy(_GroupBy):
                    chunk=chunk,
                    aggregate=_nunique_df_aggregate,
                    combine=_nunique_df_combine,
-                   meta=meta, token='series-groupby-nunique',
+                   token='series-groupby-nunique',
                    chunk_kwargs={'levels': levels, 'name': name},
                    aggregate_kwargs={'levels': levels, 'name': name},
                    combine_kwargs={'levels': levels},
@@ -1069,7 +1064,10 @@ class SeriesGroupBy(_GroupBy):
             return getattr(self, arg)(split_every=split_every,
                                       split_out=split_out)
 
-        return super(SeriesGroupBy, self).aggregate(arg, split_every=split_every, split_out=split_out)
+        result = super(SeriesGroupBy, self).aggregate(arg, split_every=split_every, split_out=split_out)
+        if self._slice:
+            result = result[self._slice]
+        return result
 
     @derived_from(pd.core.groupby.SeriesGroupBy)
     def agg(self, arg, split_every=None, split_out=1):
