@@ -130,6 +130,13 @@ def test_categorize():
         assert ddf2.index.cat.known == known_index
         assert_eq(ddf2, df.astype({'v': 'category'}), check_categorical=False)
 
+        # Specifying split_every works
+        ddf2 = ddf.categorize(index=index, split_every=2)
+        assert ddf2.y.cat.known
+        assert ddf2.v.cat.known
+        assert ddf2.index.cat.known == known_index
+        assert_eq(ddf2, df.astype({'v': 'category'}), check_categorical=False)
+
         # Specifying one column doesn't affect others
         ddf2 = ddf.categorize('v', index=index)
         assert not ddf2.y.cat.known
@@ -153,6 +160,13 @@ def test_categorize():
     assert ddf_known_index.categorize(['w']) is ddf_known_index
     assert ddf_known_index.categorize([]) is ddf_known_index
 
+    # Bad split_every fails
+    with pytest.raises(ValueError):
+        ddf.categorize(split_every=1)
+
+    with pytest.raises(ValueError):
+        ddf.categorize(split_every='foo')
+
 
 def test_categorize_index():
     # Object dtype
@@ -185,17 +199,18 @@ def test_categorical_set_index(shuffle):
     a = dd.from_pandas(df, npartitions=2)
 
     with dask.set_options(get=get_sync, shuffle=shuffle):
-        b = a.set_index('y')
+        b = a.set_index('y', npartitions=a.npartitions)
         d1, d2 = b.get_partition(0), b.get_partition(1)
         assert list(d1.index.compute()) == ['a']
         assert list(sorted(d2.index.compute())) == ['b', 'b', 'c']
 
-        b = a.set_index(a.y)
+        b = a.set_index(a.y, npartitions=a.npartitions)
         d1, d2 = b.get_partition(0), b.get_partition(1)
         assert list(d1.index.compute()) == ['a']
         assert list(sorted(d2.index.compute())) == ['b', 'b', 'c']
 
-        b = a.set_index('y', divisions=['a', 'b', 'c'])
+        b = a.set_index('y', divisions=['a', 'b', 'c'],
+                        npartitions=a.npartitions)
         d1, d2 = b.get_partition(0), b.get_partition(1)
         assert list(d1.index.compute()) == ['a']
         assert list(sorted(d2.index.compute())) == ['b', 'b', 'c']
