@@ -1,5 +1,4 @@
 import sys
-from copy import copy
 from operator import getitem, add
 from itertools import product
 
@@ -9,14 +8,18 @@ import numpy as np
 import pytest
 
 import dask
-from dask.async import get_sync
-from dask import delayed
 from dask.utils import ignoring, put_lines
 import dask.dataframe as dd
 
 from dask.dataframe.core import repartition_divisions, aca, _concat, Scalar
 from dask.dataframe.methods import boundary_slice
-from dask.dataframe.utils import assert_eq, make_meta, assert_max_deps
+from dask.dataframe.utils import (assert_eq, make_meta, assert_max_deps,
+                                  PANDAS_VERSION)
+
+if PANDAS_VERSION >= '0.20.0':
+    from pandas.io.formats import format as pandas_format
+else:
+    from pandas.formats import format as pandas_format
 
 
 dsk = {('x', 0): pd.DataFrame({'a': [1, 2, 3], 'b': [4, 5, 6]},
@@ -168,77 +171,6 @@ def test_index_names():
     assert ddf.index.compute().name == 'x'
 
 
-def test_set_index():
-    dsk = {('x', 0): pd.DataFrame({'a': [1, 2, 3], 'b': [4, 2, 6]},
-                                  index=[0, 1, 3]),
-           ('x', 1): pd.DataFrame({'a': [4, 5, 6], 'b': [3, 5, 8]},
-                                  index=[5, 6, 8]),
-           ('x', 2): pd.DataFrame({'a': [7, 8, 9], 'b': [9, 1, 8]},
-                                  index=[9, 9, 9])}
-    d = dd.DataFrame(dsk, 'x', meta, [0, 4, 9, 9])
-    full = d.compute()
-
-    d2 = d.set_index('b', npartitions=3)
-    assert d2.npartitions == 3
-    assert d2.index.name == 'b'
-    assert_eq(d2, full.set_index('b'))
-
-    d3 = d.set_index(d.b, npartitions=3)
-    assert d3.npartitions == 3
-    assert d3.index.name == 'b'
-    assert_eq(d3, full.set_index(full.b))
-
-    d4 = d.set_index('b')
-    assert d4.index.name == 'b'
-    assert_eq(d4, full.set_index('b'))
-
-
-def test_set_index_interpolate():
-    df = pd.DataFrame({'x': [4, 1, 1, 3, 3], 'y': [1., 1, 1, 1, 2]})
-    d = dd.from_pandas(df, 2)
-
-    d1 = d.set_index('x', npartitions=3)
-    assert d1.npartitions == 3
-    assert set(d1.divisions) == set([1, 2, 3, 4])
-
-    d2 = d.set_index('y', npartitions=3)
-    assert d2.divisions[0] == 1.
-    assert 1. < d2.divisions[1] < d2.divisions[2] < 2.
-    assert d2.divisions[3] == 2.
-
-
-def test_set_index_interpolate_int():
-    L = sorted(list(range(0, 200, 10)) * 2)
-    df = pd.DataFrame({'x': 2 * L})
-    d = dd.from_pandas(df, 2)
-    d1 = d.set_index('x', npartitions=10)
-    assert all(np.issubdtype(type(x), np.integer) for x in d1.divisions)
-
-
-def test_set_index_timezone():
-    s_naive = pd.Series(pd.date_range('20130101', periods=3))
-    s_aware = pd.Series(pd.date_range('20130101', periods=3, tz='US/Eastern'))
-    df = pd.DataFrame({'tz': s_aware, 'notz': s_naive})
-    d = dd.from_pandas(df, 2)
-
-    d1 = d.set_index('notz', npartitions=2)
-    s1 = pd.DatetimeIndex(s_naive.values, dtype=s_naive.dtype)
-    assert d1.divisions[0] == s_naive[0] == s1[0]
-    assert d1.divisions[-1] == s_naive[2] == s1[2]
-
-    # We currently lose "freq".  Converting data with pandas-defined dtypes
-    # to numpy or pure Python can be lossy like this.
-    d2 = d.set_index('tz', npartitions=2)
-    s2 = pd.DatetimeIndex(s_aware, dtype=s_aware.dtype)
-    assert d2.divisions[0] == s2[0]
-    assert d2.divisions[-1] == s2[2]
-    assert d2.divisions[0].tz == s2[0].tz
-    assert d2.divisions[0].tz is not None
-    s2badtype = pd.DatetimeIndex(s_aware.values, dtype=s_naive.dtype)
-    with pytest.raises(TypeError):
-        d2.divisions[0] == s2badtype[0]
-
-
 @pytest.mark.parametrize(
     'npartitions',
     [1, pytest.mark.xfail(2, reason='pandas join removes freq')]
@@ -252,48 +184,6 @@ def test_timezone_freq(npartitions):
     assert pdf.tz[0].freq == ddf.compute().tz[0].freq == ddf.tz.compute()[0].freq
 
 
-@pytest.mark.parametrize('drop', [True, False])
-def test_set_index_drop(drop):
-
-    pdf = pd.DataFrame({'A': list('ABAABBABAA'),
-                        'B': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
-                        'C': [1, 2, 3, 2, 1, 3, 2, 4, 2, 3]})
-    ddf = dd.from_pandas(pdf, 3)
-
-    assert_eq(ddf.set_index('A', drop=drop),
-              pdf.set_index('A', drop=drop))
-    assert_eq(ddf.set_index('B', drop=drop),
-              pdf.set_index('B', drop=drop))
-    assert_eq(ddf.set_index('C', drop=drop),
-              pdf.set_index('C', drop=drop))
-    assert_eq(ddf.set_index(ddf.A, drop=drop),
-              pdf.set_index(pdf.A, drop=drop))
-    assert_eq(ddf.set_index(ddf.B, drop=drop),
-              pdf.set_index(pdf.B, drop=drop))
-    assert_eq(ddf.set_index(ddf.C, drop=drop),
-              pdf.set_index(pdf.C, drop=drop))
-
-    # numeric columns
-    pdf = pd.DataFrame({0: list('ABAABBABAA'),
-                        1: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
-                        2: [1, 2, 3, 2, 1, 3, 2, 4, 2, 3]})
-    ddf = dd.from_pandas(pdf, 3)
-    assert_eq(ddf.set_index(0, drop=drop),
-              pdf.set_index(0, drop=drop))
-    assert_eq(ddf.set_index(2, drop=drop),
-              pdf.set_index(2, drop=drop))
-
-
-def test_set_index_raises_error_on_bad_input():
-    df = pd.DataFrame({'a': [1, 2, 3, 4, 5, 6, 7],
-                       'b': [7, 6, 5, 4, 3, 2, 1]})
-    ddf = dd.from_pandas(df, 2)
-
-    msg = r"Dask dataframe does not yet support multi-indexes"
-    with tm.assertRaisesRegexp(NotImplementedError, msg):
-        ddf.set_index(['a', 'b'])
-
-
 def test_rename_columns():
     # GH 819
     df = pd.DataFrame({'a': [1, 2, 3, 4, 5, 6, 7],
@@ -307,8 +197,9 @@ def test_rename_columns():
     assert_eq(ddf, df)
 
     msg = r"Length mismatch: Expected axis has 2 elements, new values have 4 elements"
-    with tm.assertRaisesRegexp(ValueError, msg):
+    with pytest.raises(ValueError) as err:
         ddf.columns = [1, 2, 3, 4]
+    assert msg in str(err.value)
 
     # Multi-index columns
     df = pd.DataFrame({('A', '0') : [1, 2, 2, 3], ('B', 1) : [1, 2, 3, 4]})
@@ -485,8 +376,8 @@ def test_where_mask():
     ddf5 = dd.from_pandas(pdf5, 2)
     pdf6 = pd.DataFrame({'a': [True, False, True] * 3,
                          'b': [False, False, True] * 3,
-                         'd': [False] * 9,
-                         'e': [True] * 9},
+                         'c': [False] * 9,
+                         'd': [True] * 9},
                         index=[5, 6, 7, 8, 9, 10, 11, 12, 13])
     ddf6 = dd.from_pandas(pdf6, 2)
 
@@ -514,11 +405,8 @@ def test_where_mask():
         assert_eq(ddf.where(ddcond, -ddf), pdf.where(pdcond, -pdf))
         assert_eq(ddf.mask(ddcond, -ddf), pdf.mask(pdcond, -pdf))
 
-        # ToDo: Should work on pandas 0.17
-        # https://github.com/pydata/pandas/pull/10283
-        # assert_eq(ddf.where(ddcond.a, -ddf), pdf.where(pdcond.a, -pdf))
-        # assert_eq(ddf.mask(ddcond.a, -ddf), pdf.mask(pdcond.a, -pdf))
-
+        assert_eq(ddf.where(ddcond.a, -ddf), pdf.where(pdcond.a, -pdf))
+        assert_eq(ddf.mask(ddcond.a, -ddf), pdf.mask(pdcond.a, -pdf))
         assert_eq(ddf.a.where(ddcond.a), pdf.a.where(pdcond.a))
         assert_eq(ddf.a.mask(ddcond.a), pdf.a.mask(pdcond.a))
         assert_eq(ddf.a.where(ddcond.a, -ddf.a), pdf.a.where(pdcond.a, -pdf.a))
@@ -657,32 +545,6 @@ def test_drop_duplicates_subset():
                       ddf.drop_duplicates(subset=ss, **kwarg))
 
 
-def test_set_partition():
-    d2 = d.set_index('b', divisions=[0, 2, 9])
-    assert d2.divisions == (0, 2, 9)
-    expected = full.set_index('b')
-    assert_eq(d2, expected)
-
-
-def test_set_partition_compute():
-    d2 = d.set_index('b', divisions=[0, 2, 9], compute=False)
-    d3 = d.set_index('b', divisions=[0, 2, 9], compute=True)
-
-    assert_eq(d2, d3)
-    assert_eq(d2, full.set_index('b'))
-    assert_eq(d3, full.set_index('b'))
-    assert len(d2.dask) > len(d3.dask)
-
-    d4 = d.set_index(d.b, divisions=[0, 2, 9], compute=False)
-    d5 = d.set_index(d.b, divisions=[0, 2, 9], compute=True)
-    exp = full.copy()
-    exp.index = exp.b
-    assert_eq(d4, d5)
-    assert_eq(d4, exp)
-    assert_eq(d5, exp)
-    assert len(d4.dask) > len(d5.dask)
-
-
 def test_get_partition():
     pdf = pd.DataFrame(np.random.randn(10, 5), columns=list('abcde'))
     ddf = dd.from_pandas(pdf, 3)
@@ -708,10 +570,10 @@ def test_get_partition():
     assert_eq(div3, pdf.a.loc[8:9])
     assert len(div1) + len(div2) + len(div3) == len(pdf.a)
 
-    with tm.assertRaises(ValueError):
+    with pytest.raises(ValueError):
         ddf.get_partition(-1)
 
-    with tm.assertRaises(ValueError):
+    with pytest.raises(ValueError):
         ddf.get_partition(3)
 
 
@@ -1027,10 +889,10 @@ def test_align_axis(join):
     assert assert_eq(res2, exp2)
 
     # invalid
-    with tm.assertRaises(ValueError):
+    with pytest.raises(ValueError):
         ddf1a.align(ddf1b, join=join, axis='XXX')
 
-    with tm.assertRaises(ValueError):
+    with pytest.raises(ValueError):
         ddf1a['A'].align(ddf1b['B'], join=join, axis=1)
 
 
@@ -1140,16 +1002,6 @@ def test_series_round():
     assert_eq(s.round(), ps.round())
 
 
-def test_set_partition_2():
-    df = pd.DataFrame({'x': [1, 2, 3, 4, 5, 6], 'y': list('abdabd')})
-    ddf = dd.from_pandas(df, 2)
-
-    result = ddf.set_index('y', divisions=['a', 'c', 'd'])
-    assert result.divisions == ('a', 'c', 'd')
-
-    assert list(result.compute(get=get_sync).index[-2:]) == ['d', 'd']
-
-
 @pytest.mark.slow
 def test_repartition():
     def _check_split_data(orig, d):
@@ -1268,24 +1120,36 @@ def test_repartition_on_pandas_dataframe():
     assert_eq(ddf, df.y)
 
 
-def test_repartition_npartitions():
-    for use_index in (True, False):
-        df = pd.DataFrame({'x': [1, 2, 3, 4, 5, 6], 'y': list('abdabd')},
-                          index=[10, 20, 30, 40, 50, 60])
-        for n in [1, 2, 4, 5]:
-            for k in [1, 2, 4, 5]:
-                if k > n:
-                    continue
-                a = dd.from_pandas(df, npartitions=n, sort=use_index)
-                k = min(a.npartitions, k)
+@pytest.mark.parametrize('use_index', [True, False])
+@pytest.mark.parametrize('n', [1, 2, 4, 5])
+@pytest.mark.parametrize('k', [1, 2, 4, 5])
+@pytest.mark.parametrize('dtype', [int, float, 'M8[ns]'])
+@pytest.mark.parametrize('transform', [lambda df: df, lambda df: df.x])
+def test_repartition_npartitions(use_index, n, k, dtype, transform):
+    df = pd.DataFrame({'x': [1, 2, 3, 4, 5, 6] * 10,
+                       'y': list('abdabd') * 10},
+                      index=pd.Series([10, 20, 30, 40, 50, 60] * 10, dtype=dtype))
+    df = transform(df)
+    a = dd.from_pandas(df, npartitions=n, sort=use_index)
+    b = a.repartition(npartitions=k)
+    assert_eq(a, b)
+    assert b.npartitions == k
+    parts = dask.get(b.dask, b._keys())
+    assert all(map(len, parts))
 
-                b = a.repartition(npartitions=k)
-                assert_eq(a, b)
-                assert b.npartitions == k
 
-        a = dd.from_pandas(df, npartitions=1)
-        with pytest.raises(ValueError):
-            a.repartition(npartitions=5)
+def test_repartition_object_index():
+    df = pd.DataFrame({'x': [1, 2, 3, 4, 5, 6] * 10},
+                      index=list('abdabd') * 10)
+    a = dd.from_pandas(df, npartitions=5)
+    b = a.repartition(npartitions=2)
+    assert b.npartitions == 2
+    assert_eq(b, df)
+
+    b = a.repartition(npartitions=10)
+    assert b.npartitions == 10
+    assert_eq(b, df)
+    assert not b.known_divisions
 
 
 @pytest.mark.slow
@@ -1304,6 +1168,19 @@ def test_repartition_freq(npartitions, freq, start, end):
     assert_eq(ddf2, df)
 
 
+def test_repartition_freq_divisions():
+    df = pd.DataFrame({'x': np.random.random(10)},
+                      index=pd.DatetimeIndex(np.random.random(10) * 100e9))
+    ddf = dd.from_pandas(df, npartitions=3)
+
+    ddf2 = ddf.repartition(freq='15s')
+    for div in ddf2.divisions[1:-1]:
+        assert div == div.round('15s')
+    assert ddf2.divisions[0] == df.index.min()
+    assert ddf2.divisions[-1] == df.index.max()
+    assert_eq(ddf2, ddf2)
+
+
 def test_repartition_freq_errors():
     df = pd.DataFrame({'x': [1, 2, 3]})
     ddf = dd.from_pandas(df, npartitions=1)
@@ -1842,10 +1719,10 @@ def test_apply():
     assert_eq(ddf.x.apply(func), df.x.apply(func))
 
     # axis=0
-    with tm.assertRaises(NotImplementedError):
+    with pytest.raises(NotImplementedError):
         ddf.apply(lambda xy: xy, axis=0)
 
-    with tm.assertRaises(NotImplementedError):
+    with pytest.raises(NotImplementedError):
         ddf.apply(lambda xy: xy, axis='index')
 
 
@@ -2058,9 +1935,9 @@ def test_nlargest_nsmallest():
         assert_eq(res2, sol)
         assert res._name != res2._name
 
-        res = f(ddf, 5, ['a', 'b'])
-        res2 = f(ddf, 5, ['a', 'b'], split_every=2)
-        sol = f(df, 5, ['a', 'b'])
+        res = f(ddf, 5, ['a', 'c'])
+        res2 = f(ddf, 5, ['a', 'c'], split_every=2)
+        sol = f(df, 5, ['a', 'c'])
         assert_eq(res, sol)
         assert_eq(res2, sol)
         assert res._name != res2._name
@@ -2165,46 +2042,6 @@ def test_groupby_callable():
               b.y.groupby(iseven).sum())
 
 
-def test_set_index_sorted_true():
-    df = pd.DataFrame({'x': [1, 2, 3, 4],
-                       'y': [10, 20, 30, 40],
-                       'z': [4, 3, 2, 1]})
-    a = dd.from_pandas(df, 2, sort=False)
-    assert not a.known_divisions
-
-    b = a.set_index('x', sorted=True)
-    assert b.known_divisions
-    assert set(a.dask).issubset(set(b.dask))
-
-    for drop in [True, False]:
-        assert_eq(a.set_index('x', drop=drop),
-                  df.set_index('x', drop=drop))
-        assert_eq(a.set_index(a.x, sorted=True, drop=drop),
-                  df.set_index(df.x, drop=drop))
-        assert_eq(a.set_index(a.x + 1, sorted=True, drop=drop),
-                  df.set_index(df.x + 1, drop=drop))
-
-    with pytest.raises(ValueError):
-        a.set_index(a.z, sorted=True)
-
-
-def test_compute_divisions():
-    from dask.dataframe.shuffle import compute_divisions
-    df = pd.DataFrame({'x': [1, 2, 3, 4],
-                       'y': [10, 20, 30, 40],
-                       'z': [4, 3, 2, 1]},
-                      index=[1, 3, 10, 20])
-    a = dd.from_pandas(df, 2, sort=False)
-    assert not a.known_divisions
-
-    divisions = compute_divisions(a)
-    b = copy(a)
-    b.divisions = divisions
-
-    assert_eq(a, b, check_divisions=False)
-    assert b.known_divisions
-
-
 def test_methods_tokenize_differently():
     df = pd.DataFrame({'x': [1, 2, 3, 4]})
     df = dd.from_pandas(df, npartitions=1)
@@ -2212,13 +2049,6 @@ def test_methods_tokenize_differently():
             df.x.map_partitions(lambda x: pd.Series(x.max()))._name)
 
 
-def test_sorted_index_single_partition():
-    df = pd.DataFrame({'x': [1, 2, 3, 4], 'y': [1, 0, 1, 0]})
-    ddf = dd.from_pandas(df, npartitions=1)
-    assert_eq(ddf.set_index('x', sorted=True),
-              df.set_index('x'))
-
-
 def _assert_info(df, ddf, memory_usage=True):
     from io import StringIO
     assert isinstance(df, pd.DataFrame)
@@ -2238,8 +2068,7 @@ def _assert_info(df, ddf, memory_usage=True):
 def test_info():
     from io import StringIO
     from dask.compatibility import unicode
-    from pandas.formats import format
-    format._put_lines = put_lines
+    pandas_format._put_lines = put_lines
 
     test_frames = [
         pd.DataFrame({'x': [1, 2, 3, 4], 'y': [1, 0, 1, 0]}, index=pd.Int64Index(range(4))),  # No RangeIndex in dask
@@ -2268,8 +2097,7 @@ def test_groupby_multilevel_info():
     from io import StringIO
     from dask.compatibility import unicode
 
-    from pandas.formats import format
-    format._put_lines = put_lines
+    pandas_format._put_lines = put_lines
 
     df = pd.DataFrame({'A': [1, 1, 2, 2],
                        'B': [1, 2, 3, 4],
@@ -2302,8 +2130,7 @@ def test_categorize_info():
     # workaround for: https://github.com/pydata/pandas/issues/14368
     from io import StringIO
     from dask.compatibility import unicode
-    from pandas.formats import format
-    format._put_lines = put_lines
+    pandas_format._put_lines = put_lines
 
     df = pd.DataFrame({'x': [1, 2, 3, 4],
                        'y': pd.Series(list('aabc')),
@@ -2443,20 +2270,6 @@ def test_getitem_multilevel():
     assert_eq(pdf[[('A', '0'), ('B', '1')]], ddf[[('A', '0'), ('B', '1')]])
 
 
-def test_set_index_sorted_min_max_same():
-    a = pd.DataFrame({'x': [1, 2, 3], 'y': [0, 0, 0]})
-    b = pd.DataFrame({'x': [1, 2, 3], 'y': [1, 1, 1]})
-
-    aa = delayed(a)
-    bb = delayed(b)
-
-    df = dd.from_delayed([aa, bb], meta=a)
-    assert not df.known_divisions
-
-    df2 = df.set_index('y', sorted=True)
-    assert df2.divisions == (0, 1, 1)
-
-
 def test_diff():
     df = pd.DataFrame(np.random.randn(100, 5), columns=list('abcde'))
     ddf = dd.from_pandas(df, 5)
@@ -2693,3 +2506,97 @@ def test_datetime_loc_open_slicing():
     assert_eq(df.loc['02.02.2015':], ddf.loc['02.02.2015':])
     assert_eq(df[0].loc[:'02.02.2015'], ddf[0].loc[:'02.02.2015'])
     assert_eq(df[0].loc['02.02.2015':], ddf[0].loc['02.02.2015':])
+
+
+def test_to_datetime():
+    df = pd.DataFrame({'year': [2015, 2016],
+                       'month': [2, 3],
+                       'day': [4, 5]})
+    ddf = dd.from_pandas(df, npartitions=2)
+
+    assert_eq(pd.to_datetime(df), dd.to_datetime(ddf))
+
+    s = pd.Series(['3/11/2000', '3/12/2000', '3/13/2000'] * 100)
+    ds = dd.from_pandas(s, npartitions=10)
+
+    assert_eq(pd.to_datetime(s, infer_datetime_format=True),
+              dd.to_datetime(ds, infer_datetime_format=True))
+
+
+@pytest.mark.parametrize('drop', [0, 9])
+def test_slice_on_filtered_boundary(drop):
+    # https://github.com/dask/dask/issues/2211
+    x = np.arange(10)
+    x[[5, 6]] -= 2
+    df = pd.DataFrame({"A": x, "B": np.arange(len(x))})
+    pdf = df.set_index("A").query("B != {}".format(drop))
+    ddf = dd.from_pandas(df, 1).set_index("A").query("B != {}".format(drop))
+
+    result = dd.concat([ddf, ddf.rename(columns={"B": "C"})], axis=1)
+    expected = pd.concat([pdf, pdf.rename(columns={"B": "C"})], axis=1)
+    assert_eq(result, expected)
+
+
+def test_boundary_slice_nonmonotonic():
+    x = np.array([-1, -2, 2, 4, 3])
+    df = pd.DataFrame({"B": range(len(x))}, index=x)
+    result = boundary_slice(df, 0, 4)
+    expected = df.iloc[2:]
+    tm.assert_frame_equal(result, expected)
+
+    result = boundary_slice(df, -1, 4)
+    expected = df.drop(-2)
+    tm.assert_frame_equal(result, expected)
+
+    result = boundary_slice(df, -2, 3)
+    expected = df.drop(4)
+    tm.assert_frame_equal(result, expected)
+
+    result = boundary_slice(df, -2, 3.5)
+    expected = df.drop(4)
+    tm.assert_frame_equal(result, expected)
+
+    result = boundary_slice(df, -2, 4)
+    expected = df
+    tm.assert_frame_equal(result, expected)
+
+
+@pytest.mark.parametrize('start, stop, right_boundary, left_boundary, drop', [
+    (-1, None, False, False, [-1, -2]),
+    (-1, None, False, True, [-2]),
+    (None, 3, False, False, [3, 4]),
+    (None, 3, True, False, [4]),
+    # Missing keys
+    (-.5, None, False, False, [-1, -2]),
+    (-.5, None, False, True, [-1, -2]),
+    (-1.5, None, False, True, [-2]),
+    (None, 3.5, False, False, [4]),
+    (None, 3.5, True, False, [4]),
+    (None, 2.5, False, False, [3, 4]),
+])
+def test_with_boundary(start, stop, right_boundary, left_boundary, drop):
+    x = np.array([-1, -2, 2, 4, 3])
+    df = pd.DataFrame({"B": range(len(x))}, index=x)
+    result = boundary_slice(df, start, stop, right_boundary, left_boundary)
+    expected = df.drop(drop)
+    tm.assert_frame_equal(result, expected)
+
+
+@pytest.mark.parametrize('index, left, right', [
+    (range(10), 0, 9),
+    (range(10), -1, None),
+    (range(10), None, 10),
+    ([-1, 0, 2, 1], None, None),
+    ([-1, 0, 2, 1], -1, None),
+    ([-1, 0, 2, 1], None, 2),
+    ([-1, 0, 2, 1], -2, 3),
+    (pd.date_range("2017", periods=10), None, None),
+    (pd.date_range("2017", periods=10), pd.Timestamp("2017"), None),
+    (pd.date_range("2017", periods=10), None, pd.Timestamp("2017-01-10")),
+    (pd.date_range("2017", periods=10), pd.Timestamp("2016"), None),
+    (pd.date_range("2017", periods=10), None, pd.Timestamp("2018")),
+])
+def test_boundary_slice_same(index, left, right):
+    df = pd.DataFrame({"A": range(len(index))}, index=index)
+    result = boundary_slice(df, left, right)
+    tm.assert_frame_equal(result, df)
