@@ -14,6 +14,8 @@ except ImportError:
 from . import base, threaded
 from .compatibility import apply
 from .core import quote
+from .context import _globals, defer_to_globals
+from .optimize import dont_optimize
 from .utils import concrete, funcname, methodcaller, ensure_dict
 from . import sharedict
 
@@ -104,14 +106,18 @@ def tokenize(*args, **kwargs):
         fails, then a unique identifier is used. If False (default), then a
         unique identifier is always used.
     """
-    if kwargs.pop('pure', False):
+    pure = kwargs.pop('pure', None)
+    if pure is None:
+        pure = _globals.get('delayed_pure', False)
+
+    if pure:
         return base.tokenize(*args, **kwargs)
     else:
         return str(uuid.uuid4())
 
 
 @curry
-def delayed(obj, name=None, pure=False, nout=None, traverse=True):
+def delayed(obj, name=None, pure=None, nout=None, traverse=True):
     """Wraps a function or object to produce a ``Delayed``.
 
     ``Delayed`` objects act as proxies for the object they wrap, but all
@@ -127,7 +133,8 @@ def delayed(obj, name=None, pure=False, nout=None, traverse=True):
     pure : bool, optional
         Indicates whether calling the resulting ``Delayed`` object is a pure
         operation. If True, arguments to the call are hashed to produce
-        deterministic keys. Default is False.
+        deterministic keys. If not provided, the default is to check the global
+        ``delayed_pure`` setting, and fallback to ``False`` if unset.
     nout : int, optional
         The number of outputs returned from calling the resulting ``Delayed``
         object. If provided, the ``Delayed`` output of the call can be iterated
@@ -165,8 +172,8 @@ def delayed(obj, name=None, pure=False, nout=None, traverse=True):
     >>> add(1, 2).compute()
     3
 
-    ``delayed`` also accepts an optional keyword ``pure``. If False (default),
-    then subsequent calls will always produce a different ``Delayed``. This is
+    ``delayed`` also accepts an optional keyword ``pure``. If False, then
+    subsequent calls will always produce a different ``Delayed``. This is
     useful for non-pure functions (such as ``time`` or ``random``).
 
     >>> from random import random
@@ -188,6 +195,21 @@ def delayed(obj, name=None, pure=False, nout=None, traverse=True):
     >>> out1.key == out2.key
     True
 
+    Instead of setting ``pure`` as a property of the callable, you can also set
+    it contextually using the ``delayed_pure`` setting. Note that this
+    influences the *call* and not the *creation* of the callable:
+
+    >>> import dask
+    >>> @delayed
+    ... def mul(a, b):
+    ...     return a * b
+    >>> with dask.set_options(delayed_pure=True):
+    ...     print(mul(1, 2).key == mul(1, 2).key)
+    True
+    >>> with dask.set_options(delayed_pure=False):
+    ...     print(mul(1, 2).key == mul(1, 2).key)
+    False
+
     The key name of the result of calling a delayed object is determined by
     hashing the arguments by default. To explicitly set the name, you can use
     the ``dask_key_name`` keyword when calling the function:
@@ -201,9 +223,9 @@ def delayed(obj, name=None, pure=False, nout=None, traverse=True):
     result. If you set the names explicitly you should make sure your key names
     are different for different results.
 
-    >>> add(1, 2, dask_key_name='three')
-    >>> add(2, 1, dask_key_name='three')
-    >>> add(2, 2, dask_key_name='four')
+    >>> add(1, 2, dask_key_name='three')  # doctest: +SKIP
+    >>> add(2, 1, dask_key_name='three')  # doctest: +SKIP
+    >>> add(2, 2, dask_key_name='four')   # doctest: +SKIP
 
     ``delayed`` can also be applied to objects to make operations on them lazy:
 
@@ -267,11 +289,14 @@ def delayed(obj, name=None, pure=False, nout=None, traverse=True):
     >>> a.count(2, pure=True).key == a.count(2, pure=True).key
     True
 
-    As with function calls, method calls also support the ``dask_key_name``
-    keyword:
+    As with function calls, method calls also respect the global
+    ``delayed_pure`` setting and support the ``dask_key_name`` keyword:
 
     >>> a.count(2, dask_key_name="count_2")
-    Delayed("count_2")
+    Delayed('count_2')
+    >>> with dask.set_options(delayed_pure=True):
+    ...     print(a.count(2).key == a.count(2).key)
+    True
     """
     if isinstance(obj, Delayed):
         return obj
@@ -322,6 +347,9 @@ def right(method):
     return _inner
 
 
+optimize = defer_to_globals('delayed_optimize', falsey=dont_optimize)(dont_optimize)
+
+
 class Delayed(base.Base):
     """Represents a value to be computed by dask.
 
@@ -330,7 +358,7 @@ class Delayed(base.Base):
     __slots__ = ('_key', 'dask', '_length')
     _finalize = staticmethod(first)
     _default_get = staticmethod(threaded.get)
-    _optimize = staticmethod(lambda d, k, **kwds: d)
+    _optimize = staticmethod(optimize)
 
     def __init__(self, key, dsk, length=None):
         self._key = key
@@ -390,7 +418,7 @@ class Delayed(base.Base):
         return self._length
 
     def __call__(self, *args, **kwargs):
-        pure = kwargs.pop('pure', False)
+        pure = kwargs.pop('pure', None)
         name = kwargs.pop('dask_key_name', None)
         func = delayed(apply, pure=pure)
         if name is not None:
@@ -410,7 +438,7 @@ class Delayed(base.Base):
     _get_unary_operator = _get_binary_operator
 
 
-def call_function(func, func_token, args, kwargs, pure=False, nout=None):
+def call_function(func, func_token, args, kwargs, pure=None, nout=None):
     dask_key_name = kwargs.pop('dask_key_name', None)
     pure = kwargs.pop('pure', pure)
 
@@ -437,7 +465,7 @@ def call_function(func, func_token, args, kwargs, pure=False, nout=None):
 class DelayedLeaf(Delayed):
     __slots__ = ('_obj', '_key', '_pure', '_nout')
 
-    def __init__(self, obj, key, pure=False, nout=None):
+    def __init__(self, obj, key, pure=None, nout=None):
         self._obj = obj
         self._key = key
         self._pure = pure
