@@ -1029,7 +1029,7 @@ class SerializableLock(object):
 
 def effective_get(get=None, collection=None):
     """Get the effective get method used in a given situation"""
-    collection_get = collection._default_get if collection else None
+    collection_get = collection._default_get if collection is not None else None
     return get or _globals.get('get') or collection_get
 
 
@@ -1053,3 +1053,25 @@ def ensure_dict(d):
             result.update(dd)
         return result
     return dict(d)
+
+
+_packages = {}
+
+
+def package_of(typ):
+    """ Return package containing type's definition
+
+    Or return None if not found
+    """
+    try:
+        return _packages[typ]
+    except KeyError:
+        # http://stackoverflow.com/questions/43462701/get-package-of-python-object/43462865#43462865
+        mod = inspect.getmodule(typ)
+        if not mod:
+            result = None
+        else:
+            base, _sep, _stem = mod.__name__.partition('.')
+            result = sys.modules[base]
+        _packages[typ] = result
+        return result
